@@ -18,6 +18,7 @@ import contextlib
 import typing
 import types
 import functools
+import pathlib
 
 from typing import Callable
 from typing import Awaitable
@@ -45,6 +46,8 @@ else:
 
 
 from . import _version
+from . import config
+from . import utils
 
 T = typing.TypeVar("T")
 U = typing.TypeVar("U")
@@ -56,9 +59,6 @@ counter = itertools.count()
 _RECEIVE_SIZE = 4096  # pretty arbitrary
 
 
-DEFAULTS = {"max_concurrent": 5}
-
-
 class TerminatedFrameReceiver:
     """Parse frames out of a Trio stream, where each frame is terminated by a
     fixed byte sequence.
@@ -221,19 +221,27 @@ def build_name_to_module(command):
     return name_to_module
 
 
-def build_function(command):
+def build_function(command, global_namespace):
     name_to_module = build_name_to_module(command)
+    global_namespace = {**name_to_module, **global_namespace}
 
     source = build_source(split_pipestring(command))
 
     local_namespace = {}
-    global_namespace = name_to_module.copy()
 
     exec(source, global_namespace, local_namespace)
     function = local_namespace["_pype_runner"]
     return function
 
 
+def build_global_namespace(source):
+    if source is None:
+        return {}
+    namespace = {}
+    exec(source, {}, namespace)
+    return namespace
+
+
 @async_generator.asynccontextmanager
 async def async_map(
     function: Callable[[T], Awaitable[U]], iterable: AsyncIterable[T], max_concurrent
@@ -339,12 +347,17 @@ async def program_runner(pairs, items, max_concurrent):
         return stack.pop_all(), items
 
 
-async def async_main(pairs, max_concurrent=DEFAULTS["max_concurrent"]):
+async def async_main(
+    pairs,
+    max_concurrent=config.DEFAULTS["max_concurrent"],
+    exec_before=config.DEFAULTS["exec_before"],
+):
     stream = trio._unix_pipes.PipeReceiveStream(os.dup(0))
     receiver = TerminatedFrameReceiver(stream, b"\n")
     result = (item.decode() async for item in receiver)
 
-    pairs = [(how, build_function(what)) for how, what in pairs]
+    global_namespace = build_global_namespace(exec_before)
+    pairs = [(how, build_function(what, global_namespace)) for how, what in pairs]
 
     stack, items = await program_runner(pairs, result, max_concurrent)
 
@@ -355,58 +368,3 @@ async def async_main(pairs, max_concurrent=DEFAULTS["max_concurrent"]):
 
 def main(pairs, **kwargs):
     trio.run(functools.partial(async_main, pairs, **kwargs))
-
-
-@click.group(chain=True)
-@click.option("--max-concurrent", type=int, default=DEFAULTS["max_concurrent"])
-@click.version_option(_version.__version__, prog_name="pype")
-def cli(**kwargs):
-    pass
-
-
-def make_subcommand(name):
-    @click.command(name)
-    @click.argument("command")
-    def _subcommand(command):
-        return (name, command)
-
-    return _subcommand
-
-
-subcommand_names = ["map", "apply", "filter", "eval", "stack"]
-subcommands = [make_subcommand(name) for name in subcommand_names]
-for subcommand in subcommands:
-    cli.add_command(subcommand)
-
-
-@cli.resultcallback()
-def collect(pairs, **kwargs):
-    main(pairs, **kwargs)
-
-
-# time python3.7 -m poetry run python -m pype map 'await asks.get(x) ! x.body ' map 'len(x)' filter 'x < 195' apply 'len(x)' <<EOF
-# http://httpbin.org/delay/3
-# http://httpbin.org/delay/1
-# http://httpbin.org/delay/1
-# http://httpbin.org/delay/1
-# http://httpbin.org/delay/1
-# http://httpbin.org/delay/1
-# http://httpbin.org/delay/1
-# http://httpbin.org/delay/1
-# http://httpbin.org/delay/1
-# EOF
-# 8
-# python3.7 -m poetry run python -m pype map 'await asks.get(x) ! x.body ' map   0.45s user 0.05s system 13% cpu 3.641 total
-
-
-# time python3.7 -m poetry run python -m pype map 'await asks.get(x) ! x.body ' map 'len(x) ! str(x) ! x[-1] ! int(x) ! x - 2 ! x * 5 ! str(x)' map '"http://httpbin.org/delay/" + x ! await asks.get(x) ! x.json() '  <<EOF
-# http://httpbin.org/delay/3
-# http://httpbin.org/delay/1
-# http://httpbin.org/delay/1
-# http://httpbin.org/delay/1
-# http://httpbin.org/delay/1
-# http://httpbin.org/delay/1
-# http://httpbin.org/delay/1
-# http://httpbin.org/delay/1
-# http://httpbin.org/delay/1
-# EOF
